import VueColor from 'vuecolor';

var Chrome = VueColor.Chrome;

var ColorPicker = {
    components: {
        'chrome-picker': Chrome,
    },
    template: `
<div class="input-group color-picker" ref="colorpicker">
  <input type="text" :name="name" class="form-control" v-model="colorValue" @focus="showPicker()" @input="updateFromInput" />
  <span class="input-group-append color-picker-container">
    <span class="current-color input-group-text" :style="'background-color: ' + colorValue" @click="togglePicker()"></span>
    <chrome-picker :value="colors" @input="updateFromPicker" v-if="displayPicker" />
  </span>
</div>`,
    props: ['color', 'name'],
    data() {
        return {
            colors: {
                hex: '#000000',
            },
            colorValue: '',
            displayPicker: false,
            firstCall: true,
        };
    },
    mounted() {
        this.setColor(this.color || '#000000');
    },
    methods: {
        setColor(color) {
            this.updateColors(color);
            this.colorValue = color;
        },
        updateColors(color) {
            if(color.slice(0, 1) == '#') {
                this.colors = {
                    hex: color
                };
            }
            else if(color.slice(0, 4) == 'rgba') {
                var rgba = color.replace(/^rgba?\(|\s+|\)$/g,'').split(','),
                    hex = '#' + ((1 << 24)  // eslint-disable-line no-bitwise
                                 + (parseInt(rgba[0]) << 16) // eslint-disable-line no-bitwise
                                 + (parseInt(rgba[1]) << 8 )  // eslint-disable-line no-bitwise
                                 + parseInt(rgba[2])).toString(16).slice(1);
                this.colors = {
                    hex: hex,
                    a: rgba[3],
                };
            }
        },
        showPicker() {
            document.addEventListener('click', this.documentClick);
            this.displayPicker = true;
        },
        hidePicker() {
            document.removeEventListener('click', this.documentClick);
            this.displayPicker = false;
        },
        togglePicker() {
            this.displayPicker ? this.hidePicker() : this.showPicker();  // eslint-disable-line babel/no-unused-expressions
        },
        updateFromInput() {
            this.updateColors(this.colorValue);
        },
        updateFromPicker(color) {
            this.colors = color;
            if(color.rgba.a == 1) {
                this.colorValue = color.hex;
            }
            else {
                this.colorValue = 'rgba(' + color.rgba.r + ', ' + color.rgba.g + ', ' + color.rgba.b + ', ' + color.rgba.a + ')';
            }
        },
        documentClick(e) {
            var el = this.$refs.colorpicker,
                target = e.target;
            if(el !== target && !el.contains(target)) {
                this.hidePicker();
            }
        }
    },
    watch: {
        colorValue(val) {
            if(val) {
                this.updateColors(val);
                if(!this.firstCall) {
                    this.$emit('input', val);
                }
                this.firstCall = false;
                //document.body.style.background = val;
            }
        }
    },
};

export default ColorPicker;
