<?php
// This file is part of Moodle - https://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <https://www.gnu.org/licenses/>.

/**
 * Form to import template
 *
 * @package     mod_otopo
 * @copyright   2022 Kosmos <moodle@kosmos.fr>
 * @license     https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
require_once("$CFG->libdir/formslib.php");

class fromtemplate_form extends moodleform {
    //Add elements to form
    public function definition() {
        global $CFG, $DB, $PAGE, $OUTPUT;

        $mform = $this->_form;

        $mform->addElement('hidden', 'id', $this->_customdata['id']);
        $mform->setType('id', PARAM_INT);

        $mform->addElement('hidden', 'action', 'import');
        $mform->setType('action', PARAM_TEXT);

        $mform->addElement('hidden', 'object', 'templates');
        $mform->setType('object', PARAM_TEXT);

        $templates = $this->_customdata['templates'];
        $templates_select = [];
        foreach($templates as $template) {
            $templates_select[$template->id] = $template->name;
        }
        $options = array(
            'multiple' => false,
            'noselectionstring' => get_string('template_choose_template', 'otopo'),
            'required' => true,
        );         
        $mform->addElement('autocomplete', 'template', get_string('template', 'otopo'), $templates_select, $options);
        $mform->addRule('template', null, 'required', null, 'server');
        $mform->setDefault('template', null);

        $this->add_action_buttons();
    }
}
?>
